/*
Play with Light
Play a piezo with light (frequency of tone) and potentiometer (duration of tone) if button
is pressed. Send values to serial port.
Components: photoresistor, potentiometer, piezo, button. 
Circuit:
- photoresistor: one terminal to 5V, one terminal to GND with 10K resistor and to analog pin A0 
- potentiometer: one side terminal to 5V, one side terminal to GND, central terminal to analog pin A1
- piezo: one terminal to GND, one terminal to digital pin 8
- button: one terminal to 5V, one terminal to GND with 10K resistor and to digital pin 2
*/


int lightPin = A0;  
int potPin = A1;  
int buttonPin = 2;
int piezoPin = 8;
int valLight = 0;
int valPot = 0;
int sensorLow = 1023;
int sensorHigh = 0;
int buttonState = LOW;
int oldButtonState = LOW;
boolean music = false;
int inByte = 0;         // incoming serial byte



void setup() {
  // open serial connection at rate in bits per second (baud) of 9600
  Serial.begin(9600);
  // set button pin as input
  pinMode(buttonPin, INPUT);
  // calibrate ambient light values
  calibrate();
  // send a byte to establish contact until receiver responds
  establishContact();
}

void loop() {
  // if connection is established
  if (Serial.available() > 0) {
    // get incoming byte and remove it from the buffer
    inByte = Serial.read();
    
    // read light
    valLight = analogRead(lightPin);
    // read potentiometer
    valPot = analogRead(potPin);
    // read button state
    buttonState = digitalRead(buttonPin);
    
    // map light value
    if (valLight < sensorLow) valLight = sensorLow;
    if (valLight > sensorHigh) valLight = sensorHigh;
    valLight = map(valLight, sensorLow, sensorHigh, 0, 255);
    // send light value
    Serial.write(valLight);
  
    // map pot value
    valPot = map(valPot, 0, 1023, 0, 255);
    // send pot value
    Serial.write(valPot);
    
    // send button values
    Serial.write(buttonState);
    
    // if press the button, play or stop music (piezo)
    if (oldButtonState == LOW && buttonState == HIGH) {
      music = !music;
    }
    // update button state
    oldButtonState = buttonState;
  
    // if music is on, play the piezo according to light and potentiometer
    if (music) {
      // frequency is light
      int frequency = map(valLight, sensorLow, sensorHigh, 50, 4000);
      // duration is potentiometer
      int duration = map(valPot, 0, 1023, 1, 500);
      tone(piezoPin, frequency, duration);
    }
  
    // wait a while to separate tones (as well as allow sync with Processing)
    delay(100); 
  }
} 

// calibrate ambient light values
void calibrate() {
  int mils = 5000;
  // turn on LED 13 during calibration
  int ledPin = 13;
  pinMode(ledPin, OUTPUT);
  digitalWrite(ledPin, HIGH);

  // calibrate for a while
  while (millis() < mils) {
    // record the maximum sensor value
    int sensorValue = analogRead(lightPin);
    if (sensorValue > sensorHigh) {
      sensorHigh = sensorValue;
    }
    // record the minimum sensor value
    if (sensorValue < sensorLow) {
      sensorLow = sensorValue;
    }
  }
  // turn LED 13 off, signaling the end of the calibration period
  digitalWrite(ledPin, LOW);
}

void establishContact() {
  while (Serial.available() <= 0) {
    Serial.print('A');   // send a capital A
    delay(300);
  }
}




