/*
Title: Move with light
Description: On Arduino, play a piezo with light (frequency of tone) and potentiometer (duration of tone).
 On Processing, draw the complex system. Speed of actors is influenced by light, color of
 background is determined by potentiometer. Press the button to show the linking network only. 
Components: photoresistor, potentiometer, piezo, button. 
Circuit:
 - photoresistor: one terminal to 5V, one terminal to GND with 10K resistor and to analog pin A0 
 - potentiometer: one side terminal to 5V, one side terminal to GND, central terminal to analog pin A1
 - piezo: one terminal to GND, one terminal to digital pin 8
 - button: one terminal to 5V, one terminal to GND with 10K resistor and to digital pin 2
Soundtrack: Rökkurró - Í Annan Heim - Svanur
 */

import processing.serial.*;
//import ddf.minim.*;
import processing.sound.*;
import java.util.Calendar;

int nCircles = 10;
Circle[] circles = {};   
Serial port;  
float val;    
float light, brightness;
int portNum = 5; // Arduino port number on my computer
int chaos = 8;
int net = 0;
int oldNet = 0;
boolean render = true;
//AudioPlayer player;
//Minim minim;
SoundFile player;
String soundtrackname = "soundtrack.mp3";
File soundtrack; 
int[] serialInArray = new int[3];    // Where we'll put what we receive
int serialCount = 0;                 // A count of how many bytes we receive
boolean firstContact = false;        // Whether we've heard from the microcontroller
boolean recording = false; // save frames for video making 


void setup() {
  size(800, 500); 
  // set lower framerate if recording
  if (recording) {
    frameRate(30);
  }
  // set full screen window
  //fullScreen();
  smooth();
  strokeWeight(3);
  colorMode(HSB, 255);
  
  // add new circles
  newCircles();
  
  // open serial connection
  port = new Serial(this, Serial.list()[portNum], 9600);
  // empty processing serial buffer
  port.clear();
  
  // play soundtrack
  soundtrack = new File(dataPath(soundtrackname));
  if (soundtrack.exists()) {
    println("Soundtrack playing...");
    //minim = new Minim(this);
    //player = minim.loadFile("soundtrack.mp3", 2048);
    //player.play();
    player = new SoundFile(this, soundtrackname);
    player.play();
  } else {
    println("No soundtrack found. I stay silent.");
  }
 if (recording) {
   println("I'm recording for video making...");
 } else {
   println("No recording.");
 }
}

void draw() {
  background(0, 0, brightness);

  for (int i=0; i < circles.length; i++) {
    // move circles
    circles[i].moveMe();
    // connect circles with a line
    circles[i].linkMe();
    // if render, draw circles
    if (render) circles[i].drawMe();
  }


  // if the button has been pressed, switch render
  if (oldNet == 0 && net == 1) {
    render = !render;
  }
  // update button state
  oldNet = net;
  
  // save image of frames for video making if recording
  if (recording) {
    saveFrame("frames/#####.tif");
  }
}

// add new circles
void newCircles() {
  for (int i=0; i < nCircles; i++) { 
    circles = (Circle[]) append(circles, new Circle(circles.length));
  }
}

// new circles when mouse is released
void mouseReleased() {
  newCircles();
}


class Circle {

  int id; // identifier
  float x, y; // coordinates of center
  float radius; // radius
  color linecol, fillcol; // colors of outline and content 
  float alpha; // trasparency
  float speed = 1; // motion speed
  float xmove = random(-speed, speed); // motion x-component   
  float ymove = random(-speed, speed); // motion y-component 

  Circle(int _id) {
    id = _id;
    x = random(width);
    y = random(height);
    radius = random(10, 100); 
    linecol = color(random(255), 255, 255);
    fillcol = color(random(255), 255, 255);
    alpha = random(255);
  }

  void drawMe() { 
    //noStroke(); 
    // set outline color
    stroke(linecol, 150);
    // set content color
    fill(fillcol, alpha);
    // draw circle
    ellipse(x, y, radius*2, radius*2);
  }

  void moveMe() {
    // move to the speed of light
    x += xmove * light;
    y += ymove * light;
    // wrap around
    if (x > (width + radius))   x = 0 - radius; 
    if (x < (0 - radius))       x = width + radius; 
    if (y > (height + radius))  y = 0 - radius; 
    if (y < (0 - radius))       y = height + radius;
  }

  void linkMe() {
    for (int i = id + 1; i < circles.length; i++) {
      float dis = dist(x, y, circles[i].x, circles[i].y); 
      float gap = dis - radius - circles[i].radius; 
      // if overlapping, connect them with a line
      if (gap < 0) { 
        stroke(0, 0, 255-brightness);
        line(x, y, circles[i].x, circles[i].y);
      }
    }
  }
}


/*
void stop()
{
  if (soundtrack.exists()) {
    player.close();
    minim.stop();
    super.stop();
  }
}
*/

void serialEvent(Serial port) {
  // read a byte from the serial port:
  int inByte = port.read();
  // if this is the first byte received, and it's an A,
  // clear the serial buffer and note that you've
  // had first contact from the microcontroller.
  // Otherwise, add the incoming byte to the array:
  if (firstContact == false) {
    if (inByte == 'A') {
      port.clear();          // clear the serial port buffer
      port.write('A');       // ask for more (send something)
      firstContact = true;   // you've had first contact from the microcontroller
    }
  } else {
    // Add the latest byte from the serial port to array:
    serialInArray[serialCount] = inByte;
    serialCount++;

    // If we have 3 bytes:
    if (serialCount > 2) {
      light = map(serialInArray[0], 0, 255, 0, chaos);
      brightness = serialInArray[1];
      net = serialInArray[2];

      // print the values (for debugging purposes only):
      //println(round(light * 10) / 10.0 + "\t" + brightness + "\t" + net);

      // Send something (a capital A) to request new sensor readings
      port.write('A');
      // Reset serialCount:
      serialCount = 0;
    }
  }
}


// safe frame is 's' is pressed
void keyReleased() {  
  if (key == 's' || key == 'S') saveFrame(timestamp()+"_####.png");
}

// get the current timestamp
String timestamp() {
  Calendar now = Calendar.getInstance();
  return String.format("%1$ty%1$tm%1$td_%1$tH%1$tM%1$tS", now);
}